@@ -3,7 +3,7 @@
 # backend for serial IO for POSIX compatible systems, like Linux, OSX
 #
 # This file is part of pySerial. https://github.com/pyserial/pyserial
-# (C) 2001-2015 Chris Liechti <cliechti@gmx.net>
+# (C) 2001-2016 Chris Liechti <cliechti@gmx.net>
 #
 # SPDX-License-Identifier:    BSD-3-Clause
 #
@@ -12,6 +12,20 @@
 #
 # references: http://www.easysw.com/~mike/serial/serial.html
 
+# Collection of port names (was previously used by number_to_device which was
+# removed.
+# - Linux                   /dev/ttyS%d (confirmed)
+# - cygwin/win32            /dev/com%d (confirmed)
+# - openbsd (OpenBSD)       /dev/cua%02d
+# - bsd*, freebsd*          /dev/cuad%d
+# - darwin (OS X)           /dev/cuad%d
+# - netbsd                  /dev/dty%02d (NetBSD 1.6 testing by Erk)
+# - irix (IRIX)             /dev/ttyf%d (partially tested) names depending on flow control
+# - hp (HP-UX)              /dev/tty%dp0 (not tested)
+# - sunos (Solaris/SunOS)   /dev/tty%c (letters, 'a'..'z') (confirmed)
+# - aix (AIX)               /dev/tty%d
+
+
 # pylint: disable=abstract-method
 import errno
 import fcntl
@@ -29,23 +43,6 @@ from serial.serialutil import SerialBase, SerialException, to_bytes, portNotOpen
 class PlatformSpecificBase(object):
     BAUDRATE_CONSTANTS = {}
 
-    def number_to_device(self, port_number):
-        sys.stderr.write("""\
-don't know how to number ttys on this system.
-! Use an explicit path (eg /dev/ttyS1) or send this information to
-! the author of this module:
-
-sys.platform = {!r}
-os.name = {!r}
-serialposix.py version = {}
-
-also add the device name of the serial port and where the
-counting starts for the first serial port.
-e.g. 'first serial port: /dev/ttyS0'
-and with a bit luck you can get this module running...
-""".format(sys.platform, os.name, serial.VERSION))
-        raise NotImplementedError('no number-to-device mapping defined on this platform')
-
     def _set_special_baudrate(self, baudrate):
         raise NotImplementedError('non-standard baudrates are not supported on this platform')
 
@@ -108,9 +105,6 @@ if plat[:5] == 'linux':    # Linux (confirmed)  # noqa
             4000000: 0o010017
         }
 
-        def number_to_device(self, port_number):
-            return '/dev/ttyS%d' % (port_number,)
-
         def _set_special_baudrate(self, baudrate):
             # right size is 44 on x86_64, allow for some growth
             buf = array.array('i', [0] * 64)
@@ -131,6 +125,7 @@ if plat[:5] == 'linux':    # Linux (confirmed)  # noqa
             buf = array.array('i', [0] * 8)  # flags, delaytx, delayrx, padding
             try:
                 fcntl.ioctl(self.fd, TIOCGRS485, buf)
+                buf[0] |= SER_RS485_ENABLED
                 if rs485_settings is not None:
                     if rs485_settings.loopback:
                         buf[0] |= SER_RS485_RX_DURING_TX
@@ -144,8 +139,8 @@ if plat[:5] == 'linux':    # Linux (confirmed)  # noqa
                         buf[0] |= SER_RS485_RTS_AFTER_SEND
                     else:
                         buf[0] &= ~SER_RS485_RTS_AFTER_SEND
-                    buf[1] = int(rs485_settings.delay_rts_before_send * 1000)
-                    buf[2] = int(rs485_settings.delay_rts_after_send * 1000)
+                    buf[1] = int(rs485_settings.delay_before_tx * 1000)
+                    buf[2] = int(rs485_settings.delay_before_rx * 1000)
                 else:
                     buf[0] = 0  # clear SER_RS485_ENABLED
                 fcntl.ioctl(self.fd, TIOCSRS485, buf)
@@ -170,28 +165,12 @@ elif plat == 'cygwin':       # cygwin/win32 (confirmed)
             3000000: 0x0100f
         }
 
-        def number_to_device(self, port_number):
-            return '/dev/com%d' % (port_number + 1,)
-
-
-elif plat[:7] == 'openbsd':    # OpenBSD
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/cua%02d' % (port_number,)
-
-elif plat[:3] == 'bsd' or plat[:7] == 'freebsd':
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/cuad%d' % (port_number,)
 
 elif plat[:6] == 'darwin':   # OS X
     import array
     IOSSIOSPEED = 0x80045402  # _IOW('T', 2, speed_t)
 
     class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/cuad%d' % (port_number,)
-
         osx_version = os.uname()[2].split('.')
         # Tiger or above can support arbitrary serial speeds
         if int(osx_version[0]) >= 8:
@@ -200,39 +179,10 @@ elif plat[:6] == 'darwin':   # OS X
                 buf = array.array('i', [baudrate])
                 fcntl.ioctl(self.fd, IOSSIOSPEED, buf, 1)
 
-
-elif plat[:6] == 'netbsd':   # NetBSD 1.6 testing by Erk
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/dty%02d' % (port_number,)
-
-elif plat[:4] == 'irix':     # IRIX (partially tested)
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/ttyf%d' % (port_number + 1,)  # XXX different device names depending on flow control
-
-elif plat[:2] == 'hp':       # HP-UX (not tested)
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/tty%dp0' % (port_number + 1,)
-
-elif plat[:5] == 'sunos':    # Solaris/SunOS (confirmed)
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/tty%c' % (ord('a') + port_number,)
-
-elif plat[:3] == 'aix':      # AIX
-    class PlatformSpecific(PlatformSpecificBase):
-        def number_to_device(self, port_number):
-            return '/dev/tty%d' % (port_number,)
-
 else:
     class PlatformSpecific(PlatformSpecificBase):
         pass
 
-# whats up with "aix", "beos", ....
-# they should work, just need to know the device names.
-
 
 # load some constants for later use.
 # try to use values from termios, use defaults from linux otherwise
@@ -308,11 +258,19 @@ class Serial(SerialBase, PlatformSpecific):
             raise
         else:
             self.is_open = True
-        if not self._dsrdtr:
-            self._update_dtr_state()
-        if not self._rtscts:
-            self._update_rts_state()
+        try:
+            if not self._dsrdtr:
+                self._update_dtr_state()
+            if not self._rtscts:
+                self._update_rts_state()
+        except IOError as e:
+            if e.errno == 22:  # ignore Invalid argument
+                pass
+            else:
+                raise
         self.reset_input_buffer()
+        self.pipe_abort_read_r, self.pipe_abort_read_w = os.pipe()
+        self.pipe_abort_write_r, self.pipe_abort_write_w = os.pipe()
 
     def _reconfigure_port(self, force_update=False):
         """Set communication parameters on opened port."""
@@ -454,6 +412,12 @@ class Serial(SerialBase, PlatformSpecific):
             if self.fd is not None:
                 os.close(self.fd)
                 self.fd = None
+                os.close(self.pipe_abort_read_w)
+                os.close(self.pipe_abort_read_r)
+                os.close(self.pipe_abort_write_w)
+                os.close(self.pipe_abort_write_r)
+                self.pipe_abort_read_r, self.pipe_abort_read_w = None, None
+                self.pipe_abort_write_r, self.pipe_abort_write_w = None, None
             self.is_open = False
 
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -479,7 +443,10 @@ class Serial(SerialBase, PlatformSpecific):
         while len(read) < size:
             try:
                 start_time = time.time()
-                ready, _, _ = select.select([self.fd], [], [], timeout)
+                ready, _, _ = select.select([self.fd, self.pipe_abort_read_r], [], [], timeout)
+                if self.pipe_abort_read_r in ready:
+                    os.read(self.pipe_abort_read_r, 1)
+                    break
                 # If select was used with a timeout, and the timeout occurs, it
                 # returns with empty lists -> thus abort read operation.
                 # For timeout == 0 (non-blocking operation) also abort when
@@ -514,31 +481,47 @@ class Serial(SerialBase, PlatformSpecific):
                     raise SerialException('read failed: {}'.format(e))
         return bytes(read)
 
+    def cancel_read(self):
+        os.write(self.pipe_abort_read_w, b"x")
+
+    def cancel_write(self):
+        os.write(self.pipe_abort_write_w, b"x")
+
     def write(self, data):
         """Output the given byte string over the serial port."""
         if not self.is_open:
             raise portNotOpenError
         d = to_bytes(data)
         tx_len = len(d)
-        if self._write_timeout is not None and self._write_timeout > 0:
-            timeout = time.time() + self._write_timeout
-        else:
-            timeout = None
+        timeout = self._write_timeout
+        if timeout and timeout > 0:   # Avoid comparing None with zero
+            timeout += time.time()
         while tx_len > 0:
             try:
                 n = os.write(self.fd, d)
-                if timeout:
+                if timeout == 0:
+                    # Zero timeout indicates non-blocking - simply return the
+                    # number of bytes of data actually written
+                    return n
+                elif timeout and timeout > 0:  # Avoid comparing None with zero
                     # when timeout is set, use select to wait for being ready
                     # with the time left as timeout
                     timeleft = timeout - time.time()
                     if timeleft < 0:
                         raise writeTimeoutError
-                    _, ready, _ = select.select([], [self.fd], [], timeleft)
+                    abort, ready, _ = select.select([self.pipe_abort_write_r], [self.fd], [], timeleft)
+                    if abort:
+                        os.read(self.pipe_abort_write_r, 1)
+                        break
                     if not ready:
                         raise writeTimeoutError
                 else:
+                    assert timeout is None
                     # wait for write operation
-                    _, ready, _ = select.select([], [self.fd], [], None)
+                    abort, ready, _ = select.select([self.pipe_abort_write_r], [self.fd], [], None)
+                    if abort:
+                        os.read(self.pipe_abort_write_r, 1)
+                        break
                     if not ready:
                         raise SerialException('write failed (select)')
                 d = d[n:]
@@ -597,19 +580,17 @@ class Serial(SerialBase, PlatformSpecific):
 
     def _update_rts_state(self):
         """Set terminal status line: Request To Send"""
-        if self._rts_state is not None:
-            if self._rts_state:
-                fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_RTS_str)
-            else:
-                fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_RTS_str)
+        if self._rts_state:
+            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_RTS_str)
+        else:
+            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_RTS_str)
 
     def _update_dtr_state(self):
         """Set terminal status line: Data Terminal Ready"""
-        if self._dtr_state is not None:
-            if self._dtr_state:
-                fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_DTR_str)
-            else:
-                fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_DTR_str)
+        if self._dtr_state:
+            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_DTR_str)
+        else:
+            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_DTR_str)
 
     @property
     def cts(self):
@@ -707,7 +688,7 @@ class PosixPollSerial(Serial):
         return less characters as requested. With no timeout it will block
         until the requested number of bytes is read.
         """
-        if self.fd is None:
+        if not self.is_open:
             raise portNotOpenError
         read = bytearray()
         poll = select.poll()
